""" Main product initializer
"""
from zope.i18nmessageid.message import MessageFactory
EEAMessageFactory = MessageFactory('eea')


def initialize(context):
    """Initializer called when used as a Zope 2 product.
    """



""" Custom setup
"""
from Products.CMFPlone.interfaces import INonInstallable
from zope.interface import implementer


@implementer(INonInstallable)
class HiddenProfiles:
    """ Hidden profiles
    """

    def getNonInstallableProfiles(self):
        """ Hide uninstall profile from site-creation and quickinstaller.
        """
        return [
            'eea.dexterity.indicators:uninstall',
            'eea.dexterity.indicators:upgrade_95',
        ]


def post_install(context):
    """ Post install script
    """
    # Do something at the end of the installation of this package.


def uninstall(context):
    """ Uninstall script
    """
    # Do something at the end of the uninstallation of this package.


def post_upgrade(context):
    """ Post upgrade script
    """
    # Do something at the end of the upgrade of this package.



""" Doc tests
"""
import doctest
import unittest
from eea.dexterity.indicators.tests.base import FUNCTIONAL_TESTING
from plone.testing import layered

OPTIONFLAGS = (doctest.REPORT_ONLY_FIRST_FAILURE |
               doctest.ELLIPSIS |
               doctest.NORMALIZE_WHITESPACE)


def test_suite():
    """ Suite
    """
    suite = unittest.TestSuite()
    suite.addTests([
        layered(
            doctest.DocFileSuite(
                'indicator.py',
                optionflags=OPTIONFLAGS,
                package='eea.dexterity.indicators.behaviors'),
            layer=FUNCTIONAL_TESTING),
        layered(
            doctest.DocFileSuite(
                'README.txt',
                optionflags=OPTIONFLAGS,
                package='eea.dexterity.indicators'),
            layer=FUNCTIONAL_TESTING),
    ])

    return suite

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



""" eea.dexterity.indicators Installer
"""
import os
from os.path import join
from setuptools import setup, find_packages

NAME = 'eea.dexterity.indicators'
PATH = NAME.split('.') + ['version.txt']
with open(join(*PATH), 'r', encoding='utf-8') as version_file:
    VERSION = version_file.read().strip()

with open("README.rst", 'r', encoding='utf-8') as readme_file:
    with open(os.path.join("docs", "HISTORY.txt"), 'r',
              encoding='utf-8') as history_file:
        LONG_DESCRIPTION = readme_file.read() + '\n' + history_file.read()

setup(
    name=NAME,
    version=VERSION,
    description="Dexterity Indicators",
    long_description_content_type="text/x-rst",
    long_description=LONG_DESCRIPTION,
    classifiers=[
        "Environment :: Web Environment",
        "Framework :: Plone",
        "Framework :: Plone :: Addon",
        "Framework :: Plone :: 4.3",
        "Framework :: Plone :: 5.2",
        "Framework :: Plone :: 6.0",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
    ],
    keywords='EEA Add-ons Plone Zope',
    author='European Environment Agency: IDM2 A-Team',
    author_email='eea-edw-a-team-alerts@googlegroups.com',
    url='https://github.com/eea/eea.dexterity.indicators',
    license='GPL version 2',
    packages=find_packages(exclude=['ez_setup']),
    namespace_packages=['eea', 'eea.dexterity'],
    include_package_data=True,
    zip_safe=False,
        install_requires=[
        'setuptools',
        # -*- Extra requirements: -*-
        'plone.app.dexterity',
        'eea.api.layout',
        'eea.schema.slate',
        'eea.dexterity.themes',
    ],
    extras_require={
        'test': [
            'plone.app.testing',
        ],
    },
    entry_points="""
    [z3c.autoinclude.plugin]
    target = plone
    """
)



""" Base test cases
"""
from Products.CMFPlone import setuphandlers
from plone.testing import z2
from plone.app.testing import TEST_USER_ID
from plone.app.testing import PloneSandboxLayer
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import setRoles
import plone.app.dexterity
import eea.api.layout
import eea.schema.slate
import eea.dexterity.indicators


class EEAFixture(PloneSandboxLayer):
    """ EEA Testing Policy
    """
    def setUpZope(self, app, configurationContext):
        """ Setup Zope
        """
        self.loadZCML(package=plone.app.dexterity)
        self.loadZCML(package=eea.api.layout)
        self.loadZCML(package=eea.schema.slate)
        self.loadZCML(package=eea.dexterity.indicators)
        z2.installProduct(app, 'eea.dexterity.indicators')

    def setUpPloneSite(self, portal):
        """ Setup Plone
        """
        applyProfile(portal, 'eea.dexterity.indicators:default')

        # Default workflow
        wftool = portal['portal_workflow']
        wftool.setDefaultChain('simple_publication_workflow')

        # Login as manager
        setRoles(portal, TEST_USER_ID, ['Manager'])

        # Add default Plone content
        try:
            applyProfile(portal, 'plone.app.contenttypes:plone-content')
        except KeyError:
            # BBB Plone 4
            setuphandlers.setupPortalContent(portal)

        # Create testing environment
        portal.invokeFactory("Folder", "sandbox", title="Sandbox")

    def tearDownZope(self, app):
        """ Uninstall Zope
        """
        z2.uninstallProduct(app, 'eea.dexterity.indicators')


EEAFIXTURE = EEAFixture()
FUNCTIONAL_TESTING = FunctionalTesting(bases=(EEAFIXTURE,),
                                       name='EEAindicators:Functional')



""" EEAContentTypes actions for plone.app.contentrules
"""

import logging
from time import time

from AccessControl import SpecialUsers, getSecurityManager
from AccessControl.SecurityManagement import (
    newSecurityManager,
    setSecurityManager,
)
from DateTime import DateTime
from OFS.SimpleItem import SimpleItem
from plone import api
from plone.app.contentrules.actions import ActionAddForm
from plone.app.contentrules.actions import ActionEditForm
from plone.app.contentrules.browser.formhelper import (
    NullAddForm,
)
from plone.contentrules.rule.interfaces import IExecutable, IRuleElementData
from zope import schema
from zope.component import adapter
from zope.interface import Interface, implementer

logger = logging.getLogger("eea.dexterity.indicators")


class IRetractAndRenameOldVersionAction(Interface):
    """Retract and rename old version"""


@implementer(IRetractAndRenameOldVersionAction, IRuleElementData)
class RetractAndRenameOldVersionAction(SimpleItem):
    """Retract and rename old version action"""

    element = "eea.dexterity.indicators.retract_and_rename_old_version"
    summary = (
        "Will retract and rename older version of this Indicator. "
        "Then rename current Indicator (remove copy_of_ from id)"
    )


@implementer(IExecutable)
@adapter(Interface, IRetractAndRenameOldVersionAction, Interface)
class RetractAndRenameOldVersionExecutor:
    """Retract and rename old version executor"""

    def __init__(self, context, element, event):
        self.context = context
        self.element = element
        self.event = event

    def __call__(self):
        obj = self.event.object
        oid = obj.getId()
        parent = obj.getParentNode()

        old_id = new_id = None
        if oid.startswith("copy_of_"):
            old_id = oid.replace("copy_of_", "", 1)
            new_id = old_id + "-%d" % time()
        elif oid.endswith(".1"):
            old_id = oid.replace(".1", "", 1)
            new_id = old_id + "-%d" % time()

        if not (old_id and new_id):
            return True

        try:
            old_version = parent[old_id]
            api.content.transition(
                obj=old_version,
                transition="markForDeletion",
                comment="Auto archive item due to new version being published",
            )

            # Bypass user roles in order to rename old version
            oldSecurityManager = getSecurityManager()
            newSecurityManager(None, SpecialUsers.system)

            api.content.rename(obj=old_version, new_id=new_id)
            api.content.rename(obj=obj, new_id=old_id)
            obj.setEffectiveDate(DateTime())
            obj.reindexObject()

            # Switch back to the current user
            setSecurityManager(oldSecurityManager)
        except Exception as err:
            logger.exception(err)
            return True
        return True


class RetractAndRenameOldVersionAddForm(NullAddForm):
    """Retract and rename old version addform"""

    def create(self):
        """Create content-rule"""
        return RetractAndRenameOldVersionAction()


class IEnableDisableDiscussionAction(Interface):
    """Enable/Disable Discussion settings schema"""

    action = schema.Choice(
        title="How discussions are changed",
        description="Should the discussions be disabled" "or enabled?",
        values=["enabled", "disabled"],
        required=True,
    )


@implementer(IEnableDisableDiscussionAction, IRuleElementData)
class EnableDisableDiscussionAction(SimpleItem):
    """Enable/Disable Discussion Action settings"""

    element = "eea.dexterity.indicators.enable_disable_discussion"
    action = None  # default value

    def summary(self):
        """Summary"""
        if self.action:
            return "Discussions will be %s" % self.action
        return "Not configured"


@implementer(IExecutable)
@adapter(Interface, IEnableDisableDiscussionAction, Interface)
class EnableDisableDiscussionActionExecutor:
    """Enable/Disable Discussion Action executor"""

    def __init__(self, context, element, event):
        self.context = context
        self.element = element
        self.event = event

    def __call__(self):
        # container = self.context
        # event = self.event
        action = self.element.action
        obj = self.event.object

        choice = {"enabled": 1, "disabled": 0}.get(action)

        if choice is None:
            return False

        if choice is not None:
            setattr(obj, "allow_discussion", bool(choice))

            logger.info(
                "Discussions for %s set to %s", obj.absolute_url(), action
            )
        else:
            logger.info(
                "eea.dexterity.indicators.actions.EnableDisable"
                "Discussion action is not properly configured"
            )
        return True


class EnableDisableDiscussionAddForm(ActionAddForm):
    """Enable/Disable Discussion addform"""

    schema = IEnableDisableDiscussionAction
    label = "Add Enable/Disable Discussion Action"
    description = "A Enable/Disable Discussion action."
    form_name = "Configure element"
    Type = EnableDisableDiscussionAction


class EnableDisableDiscussionEditForm(ActionEditForm):
    """Enable/Disable Discussion editform"""

    schema = IEnableDisableDiscussionAction
    label = "Edit Enable/Disable Discussion Action"
    description = "A Enable/Disable Discussion action."
    form_name = "Configure element"
